@@ -1,31 +1,38 @@
 package bootstrap
 
 import (
+	"context"
+
 	"github.com/gruntwork-io/terragrunt/cli/commands/common/runall"
-	"github.com/gruntwork-io/terragrunt/cli/commands/run"
 	"github.com/gruntwork-io/terragrunt/cli/flags"
+	"github.com/gruntwork-io/terragrunt/cli/flags/shared"
 	"github.com/gruntwork-io/terragrunt/internal/cli"
+	"github.com/gruntwork-io/terragrunt/internal/runner/run"
 	"github.com/gruntwork-io/terragrunt/options"
 	"github.com/gruntwork-io/terragrunt/pkg/log"
 )
 
 const CommandName = "bootstrap"
 
-func NewFlags(l log.Logger, opts *options.TerragruntOptions, prefix flags.Prefix) cli.Flags {
-	base := run.NewFlags(l, opts, prefix).Filter(run.ConfigFlagName, run.DownloadDirFlagName)
-	// Also include backend-related and feature flags explicitly for backend commands
-	base = append(base, run.NewBackendFlags(l, opts, prefix)...)
-	base = append(base, run.NewFeatureFlags(l, opts, prefix)...)
+func NewFlags(opts *options.TerragruntOptions) cli.Flags {
+	prefix := flags.Prefix{flags.TgPrefix}
 
-	return base
+	sharedFlags := cli.Flags{
+		shared.NewConfigFlag(opts, prefix, CommandName),
+		shared.NewDownloadDirFlag(opts, prefix, CommandName),
+	}
+	sharedFlags = append(sharedFlags, shared.NewBackendFlags(opts, prefix)...)
+	sharedFlags = append(sharedFlags, shared.NewFeatureFlags(opts, prefix)...)
+
+	return sharedFlags
 }
 
 func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 	cmd := &cli.Command{
 		Name:  CommandName,
 		Usage: "Bootstrap OpenTofu/Terraform backend infrastructure.",
-		Flags: NewFlags(l, opts, nil),
-		Action: func(ctx *cli.Context) error {
+		Flags: NewFlags(opts),
+		Action: func(ctx context.Context, _ *cli.Context) error {
 			return Run(ctx, l, opts.OptionsFromContext(ctx))
 		},
 	}
