@@ -3,8 +3,11 @@
 package catalog
 
 import (
+	"context"
+
 	"github.com/gruntwork-io/terragrunt/cli/commands/scaffold"
 	"github.com/gruntwork-io/terragrunt/cli/flags"
+	"github.com/gruntwork-io/terragrunt/cli/flags/shared"
 	"github.com/gruntwork-io/terragrunt/internal/cli"
 	"github.com/gruntwork-io/terragrunt/options"
 	"github.com/gruntwork-io/terragrunt/pkg/log"
@@ -15,10 +18,7 @@ const (
 )
 
 func NewFlags(opts *options.TerragruntOptions, prefix flags.Prefix) cli.Flags {
-	return scaffold.NewFlags(opts, prefix).Filter(
-		scaffold.RootFileNameFlagName,
-		scaffold.NoIncludeRootFlagName,
-	)
+	return shared.NewScaffoldingFlags(opts, prefix)
 }
 
 func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
@@ -26,10 +26,10 @@ func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 		Name:  CommandName,
 		Usage: "Launch the user interface for searching and managing your module catalog.",
 		Flags: NewFlags(opts, nil),
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, cliCtx *cli.Context) error {
 			var repoPath string
 
-			if val := ctx.Args().Get(0); val != "" {
+			if val := cliCtx.Args().Get(0); val != "" {
 				repoPath = val
 			}
 
