@@ -3,6 +3,8 @@
 package find
 
 import (
+	"context"
+
 	"github.com/gruntwork-io/terragrunt/cli/flags"
 	"github.com/gruntwork-io/terragrunt/cli/flags/shared"
 	"github.com/gruntwork-io/terragrunt/internal/cli"
@@ -91,7 +93,7 @@ func NewFlags(l log.Logger, opts *Options, prefix flags.Prefix) cli.Flags {
 			EnvVars: tgPrefix.EnvVars(External),
 			Hidden:  true,
 			Usage:   "Discover external dependencies from initial results, and add them to top-level results (implies discovery of dependencies).",
-			Action: func(_ *cli.Context, value bool) error {
+			Action: func(_ context.Context, _ *cli.Context, value bool) error {
 				if !value {
 					return nil
 				}
@@ -125,7 +127,7 @@ func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 		Aliases: []string{CommandAlias},
 		Usage:   "Find relevant Terragrunt configurations.",
 		Flags:   flags,
-		Before: func(ctx *cli.Context) error {
+		Before: func(_ context.Context, _ *cli.Context) error {
 			if cmdOpts.JSON {
 				cmdOpts.Format = FormatJSON
 			}
@@ -146,7 +148,7 @@ func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 
 			return nil
 		},
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, _ *cli.Context) error {
 			return Run(ctx, l, cmdOpts)
 		},
 	}
