@@ -1,6 +1,8 @@
 package print
 
 import (
+	"context"
+
 	"github.com/gruntwork-io/terragrunt/cli/commands/common/runall"
 	runcmd "github.com/gruntwork-io/terragrunt/cli/commands/run"
 	"github.com/gruntwork-io/terragrunt/internal/cli"
@@ -19,7 +21,7 @@ func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 		Usage:     "Print out a short description of Terragrunt context.",
 		UsageText: "terragrunt info print",
 		Flags:     runcmd.NewFlags(l, opts, nil),
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, _ *cli.Context) error {
 			return Run(ctx, l, opts)
 		},
 	}
