@@ -3,6 +3,8 @@
 package list
 
 import (
+	"context"
+
 	"github.com/gruntwork-io/terragrunt/cli/flags"
 	"github.com/gruntwork-io/terragrunt/cli/flags/shared"
 	"github.com/gruntwork-io/terragrunt/internal/cli"
@@ -32,7 +34,7 @@ const (
 	QueueConstructAsFlagAlias = "as"
 )
 
-func NewFlags(opts *Options, prefix flags.Prefix) cli.Flags {
+func NewFlags(l log.Logger, opts *Options, prefix flags.Prefix) cli.Flags {
 	tgPrefix := prefix.Prepend(flags.TgPrefix)
 
 	flags := cli.Flags{
@@ -56,10 +58,18 @@ func NewFlags(opts *Options, prefix flags.Prefix) cli.Flags {
 			Usage:       "Include dependencies in list results (only when using --long).",
 		}),
 		flags.NewFlag(&cli.BoolFlag{
-			Name:        ExternalFlagName,
-			EnvVars:     tgPrefix.EnvVars(ExternalFlagName),
-			Destination: &opts.External,
-			Usage:       "Discover external dependencies from initial results, and add them to top-level results (implies discovery of dependencies).",
+			Name:    ExternalFlagName,
+			EnvVars: tgPrefix.EnvVars(ExternalFlagName),
+			Usage:   "Discover external dependencies from initial results, and add them to top-level results (implies discovery of dependencies).",
+			Hidden:  true,
+			Action: func(_ context.Context, _ *cli.Context, value bool) error {
+				if !value {
+					return nil
+				}
+
+				opts.FilterQueries = append(opts.FilterQueries, "{./**}...")
+				return nil
+			},
 		}),
 		flags.NewFlag(&cli.BoolFlag{
 			Name:        TreeFlagName,
@@ -90,9 +100,7 @@ func NewFlags(opts *Options, prefix flags.Prefix) cli.Flags {
 		}),
 	}
 
-	flags = flags.Add(shared.NewFilterFlags(opts.TerragruntOptions)...)
-
-	return flags
+	return append(flags, shared.NewFilterFlags(l, opts.TerragruntOptions)...)
 }
 
 func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
@@ -100,7 +108,7 @@ func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 	prefix := flags.Prefix{CommandName}
 
 	// Base flags for list plus backend/feature flags
-	flags := NewFlags(cmdOpts, prefix)
+	flags := NewFlags(l, cmdOpts, prefix)
 	flags = append(flags, shared.NewBackendFlags(opts, prefix)...)
 	flags = append(flags, shared.NewFeatureFlags(opts, prefix)...)
 
@@ -109,7 +117,7 @@ func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 		Aliases: []string{CommandAlias},
 		Usage:   "List relevant Terragrunt configurations.",
 		Flags:   flags,
-		Before: func(ctx *cli.Context) error {
+		Before: func(_ context.Context, _ *cli.Context) error {
 			if cmdOpts.Tree {
 				cmdOpts.Format = FormatTree
 			}
@@ -134,7 +142,7 @@ func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 
 			return nil
 		},
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, _ *cli.Context) error {
 			return Run(ctx, l, cmdOpts)
 		},
 	}
