@@ -2,6 +2,8 @@
 package render
 
 import (
+	"context"
+
 	"github.com/gruntwork-io/terragrunt/cli/commands/common/graph"
 	"github.com/gruntwork-io/terragrunt/cli/commands/common/runall"
 	runcmd "github.com/gruntwork-io/terragrunt/cli/commands/run"
@@ -36,7 +38,7 @@ func NewFlags(opts *Options, prefix flags.Prefix) cli.Flags {
 			EnvVars:     tgPrefix.EnvVars(FormatFlagName),
 			Destination: &opts.Format,
 			Usage:       "The output format to render the config in. Currently supports: json",
-			Action: func(ctx *cli.Context, value string) error {
+			Action: func(_ context.Context, _ *cli.Context, value string) error {
 				// Set the default output path based on the format.
 				switch value {
 				case FormatJSON:
@@ -61,7 +63,7 @@ func NewFlags(opts *Options, prefix flags.Prefix) cli.Flags {
 			Name:    JSONFlagName,
 			EnvVars: tgPrefix.EnvVars(JSONFlagName),
 			Usage:   "Render the config in JSON format. Equivalent to --format=json.",
-			Action: func(ctx *cli.Context, value bool) error {
+			Action: func(_ context.Context, _ *cli.Context, value bool) error {
 				opts.Format = FormatJSON
 
 				if opts.OutputPath == "" {
@@ -122,7 +124,7 @@ func NewCommand(l log.Logger, opts *options.TerragruntOptions) *cli.Command {
 		Usage:       "Render the final terragrunt config, with all variables, includes, and functions resolved, in the specified format.",
 		Description: "This is useful for enforcing policies using static analysis tools like Open Policy Agent, or for debugging your terragrunt config.",
 		Flags:       append(runcmd.NewFlags(l, opts, nil), NewFlags(renderOpts, prefix)...),
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, _ *cli.Context) error {
 			tgOpts := opts.OptionsFromContext(ctx)
 			renderOpts := renderOpts.Clone()
 			renderOpts.TerragruntOptions = tgOpts
