@@ -12,6 +12,7 @@ import (
 	"path"
 	"path/filepath"
 	"regexp"
+	"slices"
 	"strconv"
 	"strings"
 
@@ -24,7 +25,6 @@ import (
 	"github.com/gruntwork-io/terragrunt/internal/experiment"
 	"github.com/gruntwork-io/terragrunt/internal/remotestate"
 	"github.com/gruntwork-io/terragrunt/internal/strict/controls"
-	"github.com/gruntwork-io/terragrunt/telemetry"
 
 	"github.com/mitchellh/mapstructure"
 
@@ -804,7 +804,7 @@ func (deps *ModuleDependencies) Merge(source *ModuleDependencies) {
 	}
 
 	for _, path := range source.Paths {
-		if !util.ListContainsElement(deps.Paths, path) {
+		if !slices.Contains(deps.Paths, path) {
 			deps.Paths = append(deps.Paths, path)
 		}
 	}
@@ -1140,73 +1140,86 @@ func ReadTerragruntConfig(ctx context.Context, l log.Logger, terragruntOptions *
 	l.Debugf("Reading Terragrunt config file at %s", terragruntOptions.TerragruntConfigPath)
 
 	ctx = tf.ContextWithTerraformCommandHook(ctx, nil)
-	parsingCtx := NewParsingContext(ctx, l, terragruntOptions).WithParseOption(parserOptions)
+	ctx, parsingCtx := NewParsingContext(ctx, l, terragruntOptions)
+	parsingCtx = parsingCtx.WithParseOption(parserOptions)
 
-	// TODO: Remove lint ignore
-	return ParseConfigFile(parsingCtx, l, terragruntOptions.TerragruntConfigPath, nil) //nolint:contextcheck
+	return ParseConfigFile(ctx, parsingCtx, l, terragruntOptions.TerragruntConfigPath, nil)
 }
 
 // ParseConfigFile parses the Terragrunt config file at the given path. If the include parameter is not nil, then treat this as a config
 // included in some other config file when resolving relative paths.
-func ParseConfigFile(ctx *ParsingContext, l log.Logger, configPath string, includeFromChild *IncludeConfig) (*TerragruntConfig, error) {
+func ParseConfigFile(ctx context.Context, pctx *ParsingContext, l log.Logger, configPath string, includeFromChild *IncludeConfig) (*TerragruntConfig, error) {
 	var config *TerragruntConfig
 
 	hclCache := cache.ContextCache[*hclparse.File](ctx, HclCacheContextKey)
 
-	err := telemetry.TelemeterFromContext(ctx).Collect(ctx, "parse_config_file", map[string]any{
-		"config_path": configPath,
-		"working_dir": ctx.TerragruntOptions.WorkingDir,
-	}, func(_ context.Context) error {
-		childKey := "nil"
-		if includeFromChild != nil {
-			childKey = includeFromChild.String()
+	// Build cache key components before tracing to determine cache hit status
+	childKey := "nil"
+	if includeFromChild != nil {
+		childKey = includeFromChild.String()
+	}
+
+	decodeListKey := "nil"
+	if pctx.PartialParseDecodeList != nil {
+		decodeListKey = fmt.Sprintf("%v", pctx.PartialParseDecodeList)
+	}
+
+	fileInfo, err := os.Stat(configPath)
+	if err != nil {
+		if os.IsNotExist(err) {
+			return nil, TerragruntConfigNotFoundError{Path: configPath}
 		}
 
-		decodeListKey := "nil"
-		if ctx.PartialParseDecodeList != nil {
-			decodeListKey = fmt.Sprintf("%v", ctx.PartialParseDecodeList)
-		}
+		return nil, errors.Errorf("failed to get file info: %w", err)
+	}
 
-		fileInfo, err := os.Stat(configPath)
-		if err != nil {
-			if os.IsNotExist(err) {
-				return TerragruntConfigNotFoundError{Path: configPath}
+	cacheKey := fmt.Sprintf("%v-%v-%v-%v-%v",
+		configPath,
+		pctx.TerragruntOptions.WorkingDir,
+		childKey,
+		decodeListKey,
+		fileInfo.ModTime().UnixMicro(),
+	)
+
+	// Check cache hit status before tracing
+	_, cacheHit := hclCache.Get(ctx, cacheKey)
+
+	isPartial := len(pctx.PartialParseDecodeList) > 0
+
+	err = TraceParseConfigFile(
+		ctx,
+		configPath,
+		pctx.TerragruntOptions.WorkingDir,
+		isPartial,
+		pctx.PartialParseDecodeList,
+		includeFromChild,
+		cacheHit,
+		func(childCtx context.Context) error {
+			var file *hclparse.File
+
+			if cacheConfig, found := hclCache.Get(childCtx, cacheKey); found {
+				file = cacheConfig
+			} else {
+				// Parse the HCL file into an AST body that can be decoded multiple times later without having to re-parse
+				var parseErr error
+
+				file, parseErr = hclparse.NewParser(pctx.ParserOptions...).ParseFromFile(configPath)
+				if parseErr != nil {
+					return parseErr
+				}
+
+				hclCache.Put(childCtx, cacheKey, file)
 			}
 
-			return errors.Errorf("failed to get file info: %w", err)
-		}
+			var parseErr error
 
-		cacheKey := fmt.Sprintf("%v-%v-%v-%v-%v",
-			configPath,
-			ctx.TerragruntOptions.WorkingDir,
-			childKey,
-			decodeListKey,
-			fileInfo.ModTime().UnixMicro(),
-		)
-
-		var file *hclparse.File
-
-		// TODO: Remove lint ignore
-		if cacheConfig, found := hclCache.Get(ctx, cacheKey); found { //nolint:contextcheck
-			file = cacheConfig
-		} else {
-			// Parse the HCL file into an AST body that can be decoded multiple times later without having to re-parse
-			file, err = hclparse.NewParser(ctx.ParserOptions...).ParseFromFile(configPath)
-			if err != nil {
-				return err
+			config, parseErr = ParseConfig(childCtx, pctx, l, file, includeFromChild)
+			if parseErr != nil {
+				return parseErr
 			}
-			// TODO: Remove lint ignore
-			hclCache.Put(ctx, cacheKey, file) //nolint:contextcheck
-		}
 
-		// TODO: Remove lint ignore
-		config, err = ParseConfig(ctx, l, file, includeFromChild) //nolint:contextcheck
-		if err != nil {
-			return err
-		}
-
-		return nil
-	})
+			return nil
+		})
 	if err != nil {
 		return config, err
 	}
@@ -1214,14 +1227,14 @@ func ParseConfigFile(ctx *ParsingContext, l log.Logger, configPath string, inclu
 	return config, nil
 }
 
-func ParseConfigString(ctx *ParsingContext, l log.Logger, configPath string, configString string, includeFromChild *IncludeConfig) (*TerragruntConfig, error) {
+func ParseConfigString(ctx context.Context, pctx *ParsingContext, l log.Logger, configPath string, configString string, includeFromChild *IncludeConfig) (*TerragruntConfig, error) {
 	// Parse the HCL file into an AST body that can be decoded multiple times later without having to re-parse
-	file, err := hclparse.NewParser(ctx.ParserOptions...).ParseFromString(configString, configPath)
+	file, err := hclparse.NewParser(pctx.ParserOptions...).ParseFromString(configString, configPath)
 	if err != nil {
 		return nil, err
 	}
 
-	config, err := ParseConfig(ctx, l, file, includeFromChild)
+	config, err := ParseConfig(ctx, pctx, l, file, includeFromChild)
 	if err != nil {
 		return config, err
 	}
@@ -1256,64 +1269,82 @@ func ParseConfigString(ctx *ParsingContext, l log.Logger, configPath string, con
 //  5. Merge the included config with the parsed config. Note that all the config data is mergeable except for `locals`
 //     blocks, which are only scoped to be available within the defining config.
 func ParseConfig(
-	ctx *ParsingContext,
+	ctx context.Context,
+	pctx *ParsingContext,
 	l log.Logger,
 	file *hclparse.File,
 	includeFromChild *IncludeConfig,
 ) (*TerragruntConfig, error) {
 	errs := &errors.MultiError{}
 
-	if err := DetectDeprecatedConfigurations(ctx, l, file); err != nil {
+	if err := DetectDeprecatedConfigurations(ctx, pctx, l, file); err != nil {
 		return nil, err
 	}
 
-	ctx = ctx.WithTrackInclude(nil)
+	pctx = pctx.WithTrackInclude(nil)
 
 	// Initial evaluation of configuration to load flags like IamRole which will be used for final parsing
 	// https://github.com/gruntwork-io/terragrunt/issues/667
-	if err := setIAMRole(ctx, l, file, includeFromChild); err != nil {
+	if err := setIAMRole(ctx, pctx, l, file, includeFromChild); err != nil {
 		errs = errs.Append(err)
 	}
 
 	// read unit files and add to context
-	unitValues, err := ReadValues(ctx.Context, l, ctx.TerragruntOptions, filepath.Dir(file.ConfigPath))
+	unitValues, err := ReadValues(ctx, l, pctx.TerragruntOptions, filepath.Dir(file.ConfigPath))
 	if err != nil {
 		return nil, err
 	}
 
-	ctx = ctx.WithValues(unitValues)
+	pctx = pctx.WithValues(unitValues)
 
 	// Decode just the Base blocks. See the function docs for DecodeBaseBlocks for more info on what base blocks are.
-	baseBlocks, err := DecodeBaseBlocks(ctx, l, file, includeFromChild)
+	var baseBlocks *DecodedBaseBlocks
+
+	baseBlocks, err = TraceParseBaseBlocks(ctx, l, file.ConfigPath, func(childCtx context.Context) (*DecodedBaseBlocks, error) {
+		return DecodeBaseBlocks(childCtx, pctx, l, file, includeFromChild)
+	})
 	if err != nil {
 		errs = errs.Append(err)
 	}
 
 	if baseBlocks != nil {
-		ctx = ctx.WithTrackInclude(baseBlocks.TrackInclude)
-		ctx = ctx.WithFeatures(baseBlocks.FeatureFlags)
-		ctx = ctx.WithLocals(baseBlocks.Locals)
+		pctx = pctx.WithTrackInclude(baseBlocks.TrackInclude)
+		pctx = pctx.WithFeatures(baseBlocks.FeatureFlags)
+		pctx = pctx.WithLocals(baseBlocks.Locals)
 	}
 
-	if !ctx.SkipOutputsResolution && ctx.DecodedDependencies == nil {
+	// Emit additional trace with comprehensive base blocks details
+	if baseBlocks != nil {
+		TraceParseBaseBlocksResult(ctx, file.ConfigPath, baseBlocks)
+	}
+
+	if !pctx.SkipOutputsResolution && pctx.DecodedDependencies == nil {
 		// Decode just the `dependency` blocks, retrieving the outputs from the target terragrunt config in the
 		// process.
-		retrievedOutputs, err := decodeAndRetrieveOutputs(ctx, l, file)
+		retrievedOutputs, err := decodeAndRetrieveOutputs(ctx, pctx, l, file)
 		if err != nil {
 			errs = errs.Append(err)
 		}
 
-		ctx.DecodedDependencies = retrievedOutputs
+		pctx.DecodedDependencies = retrievedOutputs
 	}
 
-	evalContext, err := createTerragruntEvalContext(ctx, l, file.ConfigPath)
+	evalContext, err := createTerragruntEvalContext(ctx, pctx, l, file.ConfigPath)
 	if err != nil {
 		errs = errs.Append(err)
 	}
 
 	// Decode the rest of the config, passing in this config's `include` block or the child's `include` block, whichever
 	// is appropriate
-	terragruntConfigFile, err := decodeAsTerragruntConfigFile(ctx, l, file, evalContext)
+	var terragruntConfigFile *terragruntConfigFile
+
+	err = TraceParseConfigDecode(ctx, file.ConfigPath, func(childCtx context.Context) error {
+		var decodeErr error
+
+		terragruntConfigFile, decodeErr = decodeAsTerragruntConfigFile(pctx, l, file, evalContext)
+
+		return decodeErr
+	})
 	if err != nil {
 		errs = errs.Append(err)
 	}
@@ -1322,15 +1353,34 @@ func ParseConfig(
 		return nil, errors.New(CouldNotResolveTerragruntConfigInFileError(file.ConfigPath))
 	}
 
-	config, err := convertToTerragruntConfig(ctx, file.ConfigPath, terragruntConfigFile)
+	config, err := convertToTerragruntConfig(ctx, pctx, file.ConfigPath, terragruntConfigFile)
 	if err != nil {
 		errs = errs.Append(err)
 	}
 
 	// If this file includes another, parse and merge it. Otherwise, just return this config.
 	// If there have been errors during this parse, don't attempt to parse the included config.
-	if ctx.TrackInclude != nil {
-		mergedConfig, err := handleInclude(ctx, l, config, false)
+	if pctx.TrackInclude != nil {
+		// Extract include paths for telemetry
+		includeCount := len(pctx.TrackInclude.CurrentList)
+		includePaths := make([]string, 0, includeCount)
+
+		for _, inc := range pctx.TrackInclude.CurrentList {
+			if inc.Path != "" {
+				includePaths = append(includePaths, inc.Path)
+			}
+		}
+
+		var mergedConfig *TerragruntConfig
+
+		err = TraceParseIncludeMerge(ctx, file.ConfigPath, includeCount, includePaths, func(childCtx context.Context) error {
+			var mergeErr error
+
+			// Use the child context for trace propagation so include parsing is a child span
+			mergedConfig, mergeErr = handleInclude(childCtx, pctx, l, config, false)
+
+			return mergeErr
+		})
 		if err != nil {
 			errs = errs.Append(err)
 			return config, errs.ErrorOrNil()
@@ -1343,7 +1393,7 @@ func ParseConfig(
 		}
 
 		// Saving processed includes into configuration, direct assignment since nested includes aren't supported
-		mergedConfig.ProcessedIncludes = ctx.TrackInclude.CurrentMap
+		mergedConfig.ProcessedIncludes = pctx.TrackInclude.CurrentMap
 		// Make sure the top level information that is not automatically merged in is captured on the merged config to
 		// ensure the proper representation of the config is captured.
 		// - Locals are deliberately not merged in so that they remain local in scope. Here, we directly set it to the
@@ -1359,7 +1409,7 @@ func ParseConfig(
 }
 
 // DetectDeprecatedConfigurations detects if deprecated configurations are used in the given HCL file.
-func DetectDeprecatedConfigurations(ctx *ParsingContext, l log.Logger, file *hclparse.File) error {
+func DetectDeprecatedConfigurations(ctx context.Context, pctx *ParsingContext, l log.Logger, file *hclparse.File) error {
 	if DetectInputsCtyUsage(file) {
 		// Dependency inputs (dependency.foo.inputs.bar) are now blocked by default for performance.
 		// This deprecated feature causes significant performance overhead due to recursive parsing.
@@ -1367,7 +1417,7 @@ func DetectDeprecatedConfigurations(ctx *ParsingContext, l log.Logger, file *hcl
 	}
 
 	if detectBareIncludeUsage(file) {
-		allControls := ctx.TerragruntOptions.StrictControls
+		allControls := pctx.TerragruntOptions.StrictControls
 
 		bareInclude := allControls.Find(controls.BareInclude)
 		if bareInclude == nil {
@@ -1473,10 +1523,10 @@ func detectBareIncludeUsage(file *hclparse.File) bool {
 var iamRoleCache = cache.NewCache[options.IAMRoleOptions](iamRoleCacheName)
 
 // setIAMRole - extract IAM role details from Terragrunt flags block
-func setIAMRole(ctx *ParsingContext, l log.Logger, file *hclparse.File, includeFromChild *IncludeConfig) error {
+func setIAMRole(ctx context.Context, pctx *ParsingContext, l log.Logger, file *hclparse.File, includeFromChild *IncludeConfig) error {
 	// Prefer the IAM Role CLI args if they were passed otherwise lazily evaluate the IamRoleOptions using the config.
-	if ctx.TerragruntOptions.OriginalIAMRoleOptions.RoleARN != "" {
-		ctx.TerragruntOptions.IAMRoleOptions = ctx.TerragruntOptions.OriginalIAMRoleOptions
+	if pctx.TerragruntOptions.OriginalIAMRoleOptions.RoleARN != "" {
+		pctx.TerragruntOptions.IAMRoleOptions = pctx.TerragruntOptions.OriginalIAMRoleOptions
 	} else {
 		// as key is considered HCL code and include configuration
 		var (
@@ -1485,7 +1535,7 @@ func setIAMRole(ctx *ParsingContext, l log.Logger, file *hclparse.File, includeF
 		)
 
 		if !found {
-			iamConfig, err := TerragruntConfigFromPartialConfig(ctx.WithDecodeList(TerragruntFlags), l, file, includeFromChild)
+			iamConfig, err := TerragruntConfigFromPartialConfig(ctx, pctx.WithDecodeList(TerragruntFlags), l, file, includeFromChild)
 			if err != nil {
 				return err
 			}
@@ -1495,16 +1545,16 @@ func setIAMRole(ctx *ParsingContext, l log.Logger, file *hclparse.File, includeF
 		}
 		// We merge the OriginalIAMRoleOptions into the one from the config, because the CLI passed IAMRoleOptions has
 		// precedence.
-		ctx.TerragruntOptions.IAMRoleOptions = options.MergeIAMRoleOptions(
+		pctx.TerragruntOptions.IAMRoleOptions = options.MergeIAMRoleOptions(
 			config,
-			ctx.TerragruntOptions.OriginalIAMRoleOptions,
+			pctx.TerragruntOptions.OriginalIAMRoleOptions,
 		)
 	}
 
 	return nil
 }
 
-func decodeAsTerragruntConfigFile(ctx *ParsingContext, l log.Logger, file *hclparse.File, evalContext *hcl.EvalContext) (*terragruntConfigFile, error) {
+func decodeAsTerragruntConfigFile(pctx *ParsingContext, l log.Logger, file *hclparse.File, evalContext *hcl.EvalContext) (*terragruntConfigFile, error) {
 	terragruntConfig := terragruntConfigFile{}
 
 	if err := file.Decode(&terragruntConfig, evalContext); err != nil {
@@ -1513,8 +1563,8 @@ func decodeAsTerragruntConfigFile(ctx *ParsingContext, l log.Logger, file *hclpa
 		ok := errors.As(err, &diagErr)
 
 		// in case of render-json command and inputs reference error, we update the inputs with default value
-		if (!ok || !isRenderJSONCommand(ctx) || !isAttributeAccessError(diagErr)) &&
-			(!ok || !isRenderCommand(ctx) || !isAttributeAccessError(diagErr)) {
+		if (!ok || !isRenderJSONCommand(pctx) || !isAttributeAccessError(diagErr)) &&
+			(!ok || !isRenderCommand(pctx) || !isAttributeAccessError(diagErr)) {
 			return &terragruntConfig, err
 		}
 
@@ -1582,11 +1632,11 @@ func getIndexOfExtraArgsWithName(extraArgs []TerraformExtraArguments, name strin
 }
 
 // Convert the contents of a fully resolved Terragrunt configuration to a TerragruntConfig object
-func convertToTerragruntConfig(ctx *ParsingContext, configPath string, terragruntConfigFromFile *terragruntConfigFile) (cfg *TerragruntConfig, err error) {
+func convertToTerragruntConfig(ctx context.Context, pctx *ParsingContext, configPath string, terragruntConfigFromFile *terragruntConfigFile) (cfg *TerragruntConfig, err error) {
 	errs := &errors.MultiError{}
 
-	if ctx.ConvertToTerragruntConfigFunc != nil {
-		return ctx.ConvertToTerragruntConfigFunc(ctx, configPath, terragruntConfigFromFile)
+	if pctx.ConvertToTerragruntConfigFunc != nil {
+		return pctx.ConvertToTerragruntConfigFunc(ctx, pctx, configPath, terragruntConfigFromFile)
 	}
 
 	terragruntConfig := &TerragruntConfig{
@@ -1631,7 +1681,7 @@ func convertToTerragruntConfig(ctx *ParsingContext, configPath string, terragrun
 		terragruntConfig.SetFieldMetadata(MetadataTerraform, defaultMetadata)
 	}
 
-	if err := validateDependencies(ctx, terragruntConfigFromFile.Dependencies); err != nil {
+	if err := validateDependencies(pctx, terragruntConfigFromFile.Dependencies); err != nil {
 		errs = errs.Append(err)
 	}
 
@@ -1800,8 +1850,8 @@ func convertToTerragruntConfig(ctx *ParsingContext, configPath string, terragrun
 		terragruntConfig.SetFieldMetadataMap(MetadataInputs, terragruntConfig.Inputs, defaultMetadata)
 	}
 
-	if ctx.Locals != nil && *ctx.Locals != cty.NilVal {
-		localsParsed, err := ctyhelper.ParseCtyValueToMap(*ctx.Locals)
+	if pctx.Locals != nil && *pctx.Locals != cty.NilVal {
+		localsParsed, err := ctyhelper.ParseCtyValueToMap(*pctx.Locals)
 		if err != nil {
 			return nil, err
 		}
