@@ -165,13 +165,13 @@ func (dep Dependency) shouldMergeMockOutputsWithState(ctx *ParsingContext) bool
 	return allowedCommand && dep.getMockOutputsMergeStrategy() != NoMerge
 }
 
-func (dep *Dependency) setRenderedOutputs(ctx *ParsingContext, l log.Logger) error {
+func (dep *Dependency) setRenderedOutputs(ctx context.Context, pctx *ParsingContext, l log.Logger) error {
 	if dep == nil {
 		return nil
 	}
 
-	if dep.shouldGetOutputs(ctx) || dep.shouldReturnMockOutputs(ctx) {
-		outputVal, err := getTerragruntOutputIfAppliedElseConfiguredDefault(ctx, l, *dep)
+	if dep.shouldGetOutputs(pctx) || dep.shouldReturnMockOutputs(pctx) {
+		outputVal, err := getTerragruntOutputIfAppliedElseConfiguredDefault(ctx, pctx, l, *dep)
 		if err != nil {
 			return err
 		}
@@ -196,8 +196,8 @@ var outputLocks = sync.Map{}
 // NOTE FOR MAINTAINER: When implementing importation of other config blocks (e.g referencing inputs), carefully
 //
 //	consider whether or not the implementation of the cyclic dependency detection still makes sense.
-func decodeAndRetrieveOutputs(ctx *ParsingContext, l log.Logger, file *hclparse.File) (*cty.Value, error) {
-	evalParsingContext, err := createTerragruntEvalContext(ctx, l, file.ConfigPath)
+func decodeAndRetrieveOutputs(ctx context.Context, pctx *ParsingContext, l log.Logger, file *hclparse.File) (*cty.Value, error) {
+	evalParsingContext, err := createTerragruntEvalContext(ctx, pctx, l, file.ConfigPath)
 	if err != nil {
 		return nil, err
 	}
@@ -222,11 +222,11 @@ func decodeAndRetrieveOutputs(ctx *ParsingContext, l log.Logger, file *hclparse.
 		}
 	}
 
-	if err := checkForDependencyBlockCycles(ctx, l, file.ConfigPath, decodedDependency); err != nil {
+	if err := checkForDependencyBlockCycles(ctx, pctx, l, file.ConfigPath, decodedDependency); err != nil {
 		return nil, err
 	}
 
-	updatedDependencies, err := decodeDependencies(ctx, l, decodedDependency)
+	updatedDependencies, err := decodeDependencies(ctx, pctx, l, decodedDependency)
 	if err != nil {
 		return nil, err
 	}
@@ -234,8 +234,8 @@ func decodeAndRetrieveOutputs(ctx *ParsingContext, l log.Logger, file *hclparse.
 	decodedDependency = *updatedDependencies
 
 	// Merge in included dependencies
-	if ctx.TrackInclude != nil {
-		mergedDecodedDependency, err := handleIncludeForDependency(ctx, l, decodedDependency)
+	if pctx.TrackInclude != nil {
+		mergedDecodedDependency, err := handleIncludeForDependency(ctx, pctx, l, decodedDependency)
 		if err != nil {
 			return nil, err
 		}
@@ -251,10 +251,10 @@ func decodeAndRetrieveOutputs(ctx *ParsingContext, l log.Logger, file *hclparse.
 
 	var result *cty.Value
 
-	err = TraceParseDependencies(ctx, file.ConfigPath, ctx.SkipOutputsResolution, len(decodedDependency.Dependencies), dependencyNames, func(childCtx context.Context) error {
+	err = TraceParseDependencies(ctx, file.ConfigPath, pctx.SkipOutputsResolution, len(decodedDependency.Dependencies), dependencyNames, func(ctx context.Context) error {
 		var depErr error
 
-		result, depErr = dependencyBlocksToCtyValue(childCtx, ctx, l, file.ConfigPath, decodedDependency.Dependencies)
+		result, depErr = dependencyBlocksToCtyValue(ctx, pctx, l, file.ConfigPath, decodedDependency.Dependencies)
 
 		return depErr
 	})
@@ -263,7 +263,7 @@ func decodeAndRetrieveOutputs(ctx *ParsingContext, l log.Logger, file *hclparse.
 }
 
 // decodeDependencies decode dependencies and fetch inputs
-func decodeDependencies(ctx *ParsingContext, l log.Logger, decodedDependency TerragruntDependency) (*TerragruntDependency, error) {
+func decodeDependencies(ctx context.Context, pctx *ParsingContext, l log.Logger, decodedDependency TerragruntDependency) (*TerragruntDependency, error) {
 	updatedDependencies := TerragruntDependency{}
 	depCache := cache.ContextCache[*dependencyOutputCache](ctx, DependencyOutputCacheContextKey)
 
@@ -277,7 +277,7 @@ func decodeDependencies(ctx *ParsingContext, l log.Logger, decodedDependency Ter
 			return &updatedDependencies, errors.New(DependencyInvalidConfigPathError{DependencyName: dep.Name})
 		}
 
-		depPath := getCleanedTargetConfigPath(dep.ConfigPath.AsString(), ctx.TerragruntOptions.TerragruntConfigPath)
+		depPath := getCleanedTargetConfigPath(dep.ConfigPath.AsString(), pctx.TerragruntOptions.TerragruntConfigPath)
 
 		if !util.FileExists(depPath) {
 			updatedDependencies.Dependencies = append(updatedDependencies.Dependencies, dep)
@@ -285,7 +285,7 @@ func decodeDependencies(ctx *ParsingContext, l log.Logger, decodedDependency Ter
 			continue
 		}
 
-		cacheKey := filepath.Join(ctx.TerragruntOptions.WorkingDir, depPath)
+		cacheKey := filepath.Join(pctx.TerragruntOptions.WorkingDir, depPath)
 
 		// Cache hit - reuse cached values
 		if cachedDependency, found := depCache.Get(ctx, cacheKey); found {
@@ -297,14 +297,14 @@ func decodeDependencies(ctx *ParsingContext, l log.Logger, decodedDependency Ter
 		}
 
 		// Cache miss - parse and cache
-		l, depOpts, err := cloneTerragruntOptionsForDependency(ctx, l, depPath)
+		l, depOpts, err := cloneTerragruntOptionsForDependency(pctx, l, depPath)
 		if err != nil {
 			return nil, err
 		}
 
-		depCtx := ctx.WithDecodeList(TerragruntFlags).WithTerragruntOptions(depOpts)
+		depCtx := pctx.WithDecodeList(TerragruntFlags).WithTerragruntOptions(depOpts).WithDiagnosticsSuppressed(l)
 
-		depConfig, err := PartialParseConfigFile(depCtx, l, depPath, nil)
+		depConfig, err := PartialParseConfigFile(ctx, depCtx, l, depPath, nil)
 		if err != nil {
 			l.Warnf("Error reading partial config for dependency %s at %s: %v", dep.Name, depPath, err)
 			updatedDependencies.Dependencies = append(updatedDependencies.Dependencies, dep)
@@ -362,7 +362,7 @@ func dependencyBlocksToModuleDependencies(l log.Logger, decodedDependencyBlocks
 
 // Check for cyclic dependency blocks to avoid infinite `terragrunt output` loops. To avoid reparsing the config, we
 // kickstart the initial loop using what we already decoded.
-func checkForDependencyBlockCycles(ctx *ParsingContext, l log.Logger, configPath string, decodedDependency TerragruntDependency) error {
+func checkForDependencyBlockCycles(ctx context.Context, pctx *ParsingContext, l log.Logger, configPath string, decodedDependency TerragruntDependency) error {
 	visitedPaths := []string{}
 	currentTraversalPaths := []string{configPath}
 
@@ -377,13 +377,13 @@ func checkForDependencyBlockCycles(ctx *ParsingContext, l log.Logger, configPath
 
 		dependencyPath := getCleanedTargetConfigPath(dependency.ConfigPath.AsString(), configPath)
 
-		l, dependencyOpts, err := cloneTerragruntOptionsForDependency(ctx, l, dependencyPath)
+		l, dependencyOpts, err := cloneTerragruntOptionsForDependency(pctx, l, dependencyPath)
 		if err != nil {
 			return err
 		}
 
-		dependencyContext := ctx.WithTerragruntOptions(dependencyOpts)
-		if err := checkForDependencyBlockCyclesUsingDFS(dependencyContext, l, dependencyPath, &visitedPaths, &currentTraversalPaths); err != nil {
+		dependencyContext := pctx.WithTerragruntOptions(dependencyOpts)
+		if err := checkForDependencyBlockCyclesUsingDFS(ctx, dependencyContext, l, dependencyPath, &visitedPaths, &currentTraversalPaths); err != nil {
 			return err
 		}
 	}
@@ -396,7 +396,8 @@ func checkForDependencyBlockCycles(ctx *ParsingContext, l log.Logger, configPath
 // Same implementation as configstack/graph.go:checkForCyclesUsingDepthFirstSearch, except walks the graph of
 // dependencies by `dependency` blocks (which make explicit `terragrunt output` calls) instead of explicit dependencies.
 func checkForDependencyBlockCyclesUsingDFS(
-	ctx *ParsingContext,
+	ctx context.Context,
+	pctx *ParsingContext,
 	l log.Logger,
 	dependencyPath string,
 	visitedPaths *[]string,
@@ -412,7 +413,7 @@ func checkForDependencyBlockCyclesUsingDFS(
 
 	*currentTraversalPaths = append(*currentTraversalPaths, dependencyPath)
 
-	dependencyPaths, err := getDependencyBlockConfigPathsByFilepath(ctx, l, dependencyPath)
+	dependencyPaths, err := getDependencyBlockConfigPathsByFilepath(ctx, pctx, l, dependencyPath)
 	if err != nil {
 		return err
 	}
@@ -420,13 +421,13 @@ func checkForDependencyBlockCyclesUsingDFS(
 	for _, dependency := range dependencyPaths {
 		dependencyPath := getCleanedTargetConfigPath(dependency, dependencyPath)
 
-		l, dependencyOpts, err := cloneTerragruntOptionsForDependency(ctx, l, dependencyPath)
+		l, dependencyOpts, err := cloneTerragruntOptionsForDependency(pctx, l, dependencyPath)
 		if err != nil {
 			return err
 		}
 
-		dependencyContext := ctx.WithTerragruntOptions(dependencyOpts)
-		if err := checkForDependencyBlockCyclesUsingDFS(dependencyContext, l, dependencyPath, visitedPaths, currentTraversalPaths); err != nil {
+		dependencyContext := pctx.WithTerragruntOptions(dependencyOpts)
+		if err := checkForDependencyBlockCyclesUsingDFS(ctx, dependencyContext, l, dependencyPath, visitedPaths, currentTraversalPaths); err != nil {
 			return err
 		}
 	}
@@ -438,12 +439,12 @@ func checkForDependencyBlockCyclesUsingDFS(
 }
 
 // Given the config path, return the list of config paths that are specified as dependency blocks in the config
-func getDependencyBlockConfigPathsByFilepath(ctx *ParsingContext, l log.Logger, configPath string) ([]string, error) {
+func getDependencyBlockConfigPathsByFilepath(ctx context.Context, pctx *ParsingContext, l log.Logger, configPath string) ([]string, error) {
 	// This will automatically parse everything needed to parse the dependency block configs, and load them as
 	// TerragruntConfig.Dependencies. Note that since we aren't passing in `DependenciesBlock` to the
 	// PartialDecodeSectionType list, the Dependencies attribute will not include any dependencies specified via the
 	// dependencies block.
-	tgConfig, err := PartialParseConfigFile(ctx.WithDecodeList(DependencyBlock), l, configPath, nil)
+	tgConfig, err := PartialParseConfigFile(ctx, pctx.WithDecodeList(DependencyBlock).WithDiagnosticsSuppressed(l), l, configPath, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -463,7 +464,7 @@ func getDependencyBlockConfigPathsByFilepath(ctx *ParsingContext, l log.Logger,
 // This routine will go through the process of obtaining the outputs using `terragrunt output` from the target config.
 // The traceCtx parameter is the trace context from the parent span (parse_dependencies) to establish parent-child
 // relationship for individual dependency traces.
-func dependencyBlocksToCtyValue(traceCtx context.Context, parsingCtx *ParsingContext, l log.Logger, parentConfigPath string, dependencyConfigs []Dependency) (*cty.Value, error) {
+func dependencyBlocksToCtyValue(traceCtx context.Context, pctx *ParsingContext, l log.Logger, parentConfigPath string, dependencyConfigs []Dependency) (*cty.Value, error) {
 	paths := []string{}
 
 	// dependencyMap is the top level map that maps dependency block names to the encoded version, which includes
@@ -482,13 +483,13 @@ func dependencyBlocksToCtyValue(traceCtx context.Context, parsingCtx *ParsingCon
 			}
 
 			// Use traceCtx to make this a child span of parse_dependencies
-			return TraceParseDependency(traceCtx, dependencyConfig.Name, depPath, func(_ context.Context) error {
+			return TraceParseDependency(traceCtx, dependencyConfig.Name, depPath, func(ctx context.Context) error {
 				// Loose struct to hold the attributes of the dependency. This includes:
 				// - outputs: The module outputs of the target config
 				dependencyEncodingMap := map[string]cty.Value{}
 
 				// Encode the outputs and nest under `outputs` attribute if we should get the outputs or the `mock_outputs`
-				if err := dependencyConfig.setRenderedOutputs(parsingCtx, l); err != nil { //nolint:contextcheck
+				if err := dependencyConfig.setRenderedOutputs(ctx, pctx, l); err != nil {
 					return err
 				}
 
@@ -544,19 +545,19 @@ func dependencyBlocksToCtyValue(traceCtx context.Context, parsingCtx *ParsingCon
 //   - If the dependency block indicates a mock_outputs attribute, this will return that.
 //     If the dependency block indicates a mock_outputs_merge_strategy_with_state attribute, mock_outputs and state outputs will be merged following the merge strategy
 //   - If the dependency block does NOT indicate a mock_outputs attribute, this will return an error.
-func getTerragruntOutputIfAppliedElseConfiguredDefault(ctx *ParsingContext, l log.Logger, dependencyConfig Dependency) (*cty.Value, error) {
+func getTerragruntOutputIfAppliedElseConfiguredDefault(ctx context.Context, pctx *ParsingContext, l log.Logger, dependencyConfig Dependency) (*cty.Value, error) {
 	if dependencyConfig.isDisabled() {
 		l.Debugf("Skipping outputs reading for disabled dependency %s", dependencyConfig.Name)
 		return dependencyConfig.MockOutputs, nil
 	}
 
-	if dependencyConfig.shouldGetOutputs(ctx) {
-		outputVal, isEmpty, err := getTerragruntOutput(ctx, l, dependencyConfig)
+	if dependencyConfig.shouldGetOutputs(pctx) {
+		outputVal, isEmpty, err := getTerragruntOutput(ctx, pctx, l, dependencyConfig)
 		if err != nil {
 			return nil, err
 		}
 
-		if !isEmpty && dependencyConfig.shouldMergeMockOutputsWithState(ctx) && dependencyConfig.MockOutputs != nil {
+		if !isEmpty && dependencyConfig.shouldMergeMockOutputsWithState(pctx) && dependencyConfig.MockOutputs != nil {
 			mockMergeStrategy := dependencyConfig.getMockOutputsMergeStrategy()
 
 			// TODO: Make this exhaustive
@@ -578,12 +579,12 @@ func getTerragruntOutputIfAppliedElseConfiguredDefault(ctx *ParsingContext, l lo
 	// When we get no output, it can be an indication that either the module has no outputs or the module is not
 	// applied. In either case, check if there are default output values to return. If yes, return that. Else,
 	// return error.
-	targetConfig := getCleanedTargetConfigPath(dependencyConfig.ConfigPath.AsString(), ctx.TerragruntOptions.TerragruntConfigPath)
+	targetConfig := getCleanedTargetConfigPath(dependencyConfig.ConfigPath.AsString(), pctx.TerragruntOptions.TerragruntConfigPath)
 
-	if dependencyConfig.shouldReturnMockOutputs(ctx) {
+	if dependencyConfig.shouldReturnMockOutputs(pctx) {
 		l.Warnf("Config %s is a dependency of %s that has no outputs, but mock outputs provided and returning those in dependency output.",
 			targetConfig,
-			ctx.TerragruntOptions.TerragruntConfigPath,
+			pctx.TerragruntOptions.TerragruntConfigPath,
 		)
 
 		return dependencyConfig.MockOutputs, nil
@@ -596,7 +597,7 @@ func getTerragruntOutputIfAppliedElseConfiguredDefault(ctx *ParsingContext, l lo
 		targetName:    dependencyConfig.Name,
 		targetPath:    dependencyConfig.ConfigPath.AsString(),
 		targetConfig:  targetConfig,
-		currentConfig: ctx.TerragruntOptions.TerragruntConfigPath,
+		currentConfig: pctx.TerragruntOptions.TerragruntConfigPath,
 	}
 
 	return nil, err
@@ -604,7 +605,7 @@ func getTerragruntOutputIfAppliedElseConfiguredDefault(ctx *ParsingContext, l lo
 
 // We should only return default outputs if the mock_outputs attribute is set, and if we are running one of the
 // allowed commands when `mock_outputs_allowed_terraform_commands` is set as well.
-func (dep Dependency) shouldReturnMockOutputs(ctx *ParsingContext) bool {
+func (dep Dependency) shouldReturnMockOutputs(pctx *ParsingContext) bool {
 	if dep.isDisabled() {
 		return true
 	}
@@ -614,29 +615,29 @@ func (dep Dependency) shouldReturnMockOutputs(ctx *ParsingContext) bool {
 	allowedCommand :=
 		dep.MockOutputsAllowedTerraformCommands == nil ||
 			len(*dep.MockOutputsAllowedTerraformCommands) == 0 ||
-			slices.Contains(*dep.MockOutputsAllowedTerraformCommands, ctx.TerragruntOptions.OriginalTerraformCommand)
+			slices.Contains(*dep.MockOutputsAllowedTerraformCommands, pctx.TerragruntOptions.OriginalTerraformCommand)
 
-	return defaultOutputsSet && allowedCommand || isRenderJSONCommand(ctx) || isRenderCommand(ctx)
+	return defaultOutputsSet && allowedCommand || isRenderJSONCommand(pctx) || isRenderCommand(pctx)
 }
 
 // Return the output from the state of another module, managed by terragrunt. This function will parse the provided
 // terragrunt config and extract the desired output from the remote state. Note that this will error if the targeted
 // module hasn't been applied yet.
-func getTerragruntOutput(ctx *ParsingContext, l log.Logger, dependencyConfig Dependency) (*cty.Value, bool, error) {
+func getTerragruntOutput(ctx context.Context, pctx *ParsingContext, l log.Logger, dependencyConfig Dependency) (*cty.Value, bool, error) {
 	// target config check: make sure the target config exists
-	targetConfigPath := getCleanedTargetConfigPath(dependencyConfig.ConfigPath.AsString(), ctx.TerragruntOptions.TerragruntConfigPath)
+	targetConfigPath := getCleanedTargetConfigPath(dependencyConfig.ConfigPath.AsString(), pctx.TerragruntOptions.TerragruntConfigPath)
 
 	if !util.FileExists(targetConfigPath) {
 		return nil, true, errors.New(DependencyConfigNotFound{Path: targetConfigPath})
 	}
 
-	jsonBytes, err := getOutputJSONWithCaching(ctx, l, targetConfigPath)
+	jsonBytes, err := getOutputJSONWithCaching(ctx, pctx, l, targetConfigPath)
 	if err != nil {
-		if !isRenderJSONCommand(ctx) && !isRenderCommand(ctx) && !isAwsS3NoSuchKey(err) {
+		if !isRenderJSONCommand(pctx) && !isRenderCommand(pctx) && !isAwsS3NoSuchKey(err) {
 			return nil, true, err
 		}
 
-		l.Warnf("Failed to read outputs from %s referenced in %s as %s, fallback to mock outputs. Error: %v", targetConfigPath, ctx.TerragruntOptions.TerragruntConfigPath, dependencyConfig.Name, err)
+		l.Warnf("Failed to read outputs from %s referenced in %s as %s, fallback to mock outputs. Error: %v", targetConfigPath, pctx.TerragruntOptions.TerragruntConfigPath, dependencyConfig.Name, err)
 
 		jsonBytes, err = json.Marshal(dependencyConfig.MockOutputs)
 		if err != nil {
@@ -670,17 +671,17 @@ func isAwsS3NoSuchKey(err error) bool {
 }
 
 // isRenderJSONCommand This function will true if terragrunt was invoked with render-json
-func isRenderJSONCommand(ctx *ParsingContext) bool {
-	return slices.Contains(ctx.TerragruntOptions.TerraformCliArgs, renderJSONCommand)
+func isRenderJSONCommand(pctx *ParsingContext) bool {
+	return slices.Contains(pctx.TerragruntOptions.TerraformCliArgs, renderJSONCommand)
 }
 
 // isRenderCommand will return true if terragrunt was invoked with render
-func isRenderCommand(ctx *ParsingContext) bool {
-	return slices.Contains(ctx.TerragruntOptions.TerraformCliArgs, renderCommand)
+func isRenderCommand(pctx *ParsingContext) bool {
+	return slices.Contains(pctx.TerragruntOptions.TerraformCliArgs, renderCommand)
 }
 
 // getOutputJSONWithCaching will run terragrunt output on the target config if it is not already cached.
-func getOutputJSONWithCaching(ctx *ParsingContext, l log.Logger, targetConfig string) ([]byte, error) {
+func getOutputJSONWithCaching(ctx context.Context, pctx *ParsingContext, l log.Logger, targetConfig string) ([]byte, error) {
 	// Acquire synchronization lock to ensure only one instance of output is called per config.
 	rawActualLock, _ := outputLocks.LoadOrStore(targetConfig, &sync.Mutex{})
 
@@ -692,7 +693,7 @@ func getOutputJSONWithCaching(ctx *ParsingContext, l log.Logger, targetConfig st
 	// This debug log is useful for validating if the locking mechanism is working. If the locking mechanism is working,
 	// we should only see one pair of logs at a time that begin with this statement, and then the relevant "terraform
 	// output" log for the dependency.
-	l.Debugf("Getting output of dependency %s for config %s", targetConfig, ctx.TerragruntOptions.TerragruntConfigPath)
+	l.Debugf("Getting output of dependency %s for config %s", targetConfig, pctx.TerragruntOptions.TerragruntConfigPath)
 
 	// Look up if we have already run terragrunt output for this target config
 	rawJSONBytes, hasRun := jsonOutputCache.Load(targetConfig)
@@ -703,7 +704,7 @@ func getOutputJSONWithCaching(ctx *ParsingContext, l log.Logger, targetConfig st
 	}
 
 	// Cache miss, so look up the output and store in cache
-	newJSONBytes, err := getTerragruntOutputJSON(ctx, l, targetConfig)
+	newJSONBytes, err := getTerragruntOutputJSON(ctx, pctx, l, targetConfig)
 	if err != nil {
 		return nil, err
 	}
@@ -726,8 +727,8 @@ func getOutputJSONWithCaching(ctx *ParsingContext, l log.Logger, targetConfig st
 // - The original config path to the dependency module's config
 //
 // That way, everything in that dependency happens within its own ctx.
-func cloneTerragruntOptionsForDependency(ctx *ParsingContext, l log.Logger, targetConfigPath string) (log.Logger, *options.TerragruntOptions, error) {
-	l, targetOptions, err := ctx.TerragruntOptions.CloneWithConfigPath(l, targetConfigPath)
+func cloneTerragruntOptionsForDependency(pctx *ParsingContext, l log.Logger, targetConfigPath string) (log.Logger, *options.TerragruntOptions, error) {
+	l, targetOptions, err := pctx.TerragruntOptions.CloneWithConfigPath(l, targetConfigPath)
 	if err != nil {
 		return l, nil, err
 	}
@@ -748,8 +749,8 @@ func cloneTerragruntOptionsForDependency(ctx *ParsingContext, l log.Logger, targ
 }
 
 // Clone terragrunt options and update ctx for dependency block so that the outputs can be read correctly
-func cloneTerragruntOptionsForDependencyOutput(ctx *ParsingContext, l log.Logger, targetConfig string) (log.Logger, *options.TerragruntOptions, error) {
-	l, targetOptions, err := cloneTerragruntOptionsForDependency(ctx, l, targetConfig)
+func cloneTerragruntOptionsForDependencyOutput(ctx context.Context, pctx *ParsingContext, l log.Logger, targetConfig string) (log.Logger, *options.TerragruntOptions, error) {
+	l, targetOptions, err := cloneTerragruntOptionsForDependency(pctx, l, targetConfig)
 	if err != nil {
 		return l, nil, err
 	}
@@ -771,10 +772,11 @@ func cloneTerragruntOptionsForDependencyOutput(ctx *ParsingContext, l log.Logger
 
 	targetOptions.DownloadDir = downloadDir
 
-	targetParsingContext := ctx.WithTerragruntOptions(targetOptions)
+	targetParsingContext := pctx.WithTerragruntOptions(targetOptions)
 	// Validate and use TerragruntVersionConstraints.TerraformBinary for dependency
 	partialTerragruntConfig, err := PartialParseConfigFile(
-		targetParsingContext.WithDecodeList(DependencyBlock),
+		ctx,
+		targetParsingContext.WithDecodeList(DependencyBlock).WithDiagnosticsSuppressed(l),
 		l,
 		targetConfig,
 		nil,
@@ -789,10 +791,11 @@ func cloneTerragruntOptionsForDependencyOutput(ctx *ParsingContext, l log.Logger
 	}
 
 	// If the Source is set, then we need to recompute it in the ctx of the target config.
-	if ctx.TerragruntOptions.Source != "" {
+	if pctx.TerragruntOptions.Source != "" {
 		// We need the terraform source of the target config to compute the actual source to use
 		partialParseIncludedConfig, err := PartialParseConfigFile(
-			targetParsingContext.WithDecodeList(TerraformBlock),
+			ctx,
+			targetParsingContext.WithDecodeList(TerraformBlock).WithDiagnosticsSuppressed(l),
 			l,
 			targetConfig,
 			nil,
@@ -801,7 +804,7 @@ func cloneTerragruntOptionsForDependencyOutput(ctx *ParsingContext, l log.Logger
 			return l, nil, err
 		}
 		// Update the source value to be everything before "//" so that it can be recomputed
-		moduleURL, _ := getter.SourceDirSubdir(ctx.TerragruntOptions.Source)
+		moduleURL, _ := getter.SourceDirSubdir(pctx.TerragruntOptions.Source)
 
 		// Finally, update the source to be the combined path between the terraform source in the target config, and the
 		// value before "//" in the original terragrunt options.
@@ -823,25 +826,26 @@ func cloneTerragruntOptionsForDependencyOutput(ctx *ParsingContext, l log.Logger
 // If these conditions are met, terragrunt can optimize the retrieval to avoid recursively retrieving dependency outputs
 // by directly pulling down the state file. Otherwise, terragrunt will fallback to running `terragrunt output` on the
 // target module.
-func getTerragruntOutputJSON(ctx *ParsingContext, l log.Logger, targetConfig string) ([]byte, error) {
+func getTerragruntOutputJSON(ctx context.Context, pctx *ParsingContext, l log.Logger, targetConfig string) ([]byte, error) {
 	// Make a copy of the terragruntOptions so that we can reuse the same execution environment, but in the ctx of
 	// the target config.
-	l, targetTGOptions, err := cloneTerragruntOptionsForDependencyOutput(ctx, l, targetConfig)
+	l, targetTGOptions, err := cloneTerragruntOptionsForDependencyOutput(ctx, pctx, l, targetConfig)
 	if err != nil {
 		return nil, err
 	}
 
-	ctx = ctx.WithTerragruntOptions(targetTGOptions)
+	pctx = pctx.WithTerragruntOptions(targetTGOptions)
 
 	// First attempt to parse the `remote_state` blocks without parsing/getting dependency outputs. If this is possible,
 	// proceed to routine that fetches remote state directly. Otherwise, fallback to calling `terragrunt output`
 	// directly.
 
 	// we need to suspend logging diagnostic errors on this attempt
-	parseOptions := append(ctx.ParserOptions, hclparse.WithDiagnosticsWriter(io.Discard, true))
+	parseOptions := append(pctx.ParserOptions, hclparse.WithDiagnosticsWriter(io.Discard, true))
 
 	remoteStateTGConfig, err := PartialParseConfigFile(
-		ctx.WithParseOption(parseOptions).WithDecodeList(
+		ctx,
+		pctx.WithParseOption(parseOptions).WithDecodeList(
 			RemoteStateBlock,
 			TerragruntFlags,
 			EngineBlock,
@@ -853,7 +857,7 @@ func getTerragruntOutputJSON(ctx *ParsingContext, l log.Logger, targetConfig str
 	canGet := canGetRemoteState(remoteStateTGConfig.RemoteState)
 
 	if err != nil || !canGet {
-		l, targetOpts, err := cloneTerragruntOptionsForDependency(ctx, l, targetConfig)
+		l, targetOpts, err := cloneTerragruntOptionsForDependency(pctx, l, targetConfig)
 		if err != nil {
 			return nil, err
 		}
@@ -861,12 +865,12 @@ func getTerragruntOutputJSON(ctx *ParsingContext, l log.Logger, targetConfig str
 		l.Debugf("Could not parse remote_state block from target config %s", targetOpts.TerragruntConfigPath)
 		l.Debugf("Falling back to terragrunt output.")
 
-		return runTerragruntOutputJSON(ctx, l, targetConfig)
+		return runTerragruntOutputJSON(ctx, pctx, l, targetConfig)
 	}
 
 	// In optimization mode, see if there is already an init-ed folder that terragrunt can use, and if so, run
 	// `terraform output` in the working directory.
-	isInit, workingDir, err := terragruntAlreadyInit(l, targetTGOptions, targetConfig, ctx)
+	isInit, workingDir, err := terragruntAlreadyInit(ctx, l, targetTGOptions, targetConfig, pctx)
 	if err != nil {
 		return nil, err
 	}
@@ -877,14 +881,15 @@ func getTerragruntOutputJSON(ctx *ParsingContext, l log.Logger, targetConfig str
 		return nil, err
 	}
 
-	ctx.TerragruntOptions.Engine = engineOpts
+	pctx.TerragruntOptions.Engine = engineOpts
 
 	if isInit {
-		return getTerragruntOutputJSONFromInitFolder(ctx, l, workingDir, remoteStateTGConfig.GetIAMRoleOptions())
+		return getTerragruntOutputJSONFromInitFolder(ctx, pctx, l, workingDir, remoteStateTGConfig.GetIAMRoleOptions())
 	}
 
 	return getTerragruntOutputJSONFromRemoteState(
 		ctx,
+		pctx,
 		l,
 		targetConfig,
 		remoteStateTGConfig.RemoteState,
@@ -900,10 +905,10 @@ func canGetRemoteState(remoteState *remotestate.RemoteState) bool {
 // terragruntAlreadyInit returns true if it detects that the module specified by the given terragrunt configuration is
 // already initialized with the terraform source. This will also return the working directory where you can run
 // terraform.
-func terragruntAlreadyInit(l log.Logger, opts *options.TerragruntOptions, configPath string, ctx *ParsingContext) (bool, string, error) {
+func terragruntAlreadyInit(ctx context.Context, l log.Logger, opts *options.TerragruntOptions, configPath string, pctx *ParsingContext) (bool, string, error) {
 	// We need to first determine the working directory where the terraform source should be located. This is dependent
 	// on the source field of the terraform block in the config.
-	terraformBlockTGConfig, err := PartialParseConfigFile(ctx.WithDecodeList(TerraformSource), l, configPath, nil)
+	terraformBlockTGConfig, err := PartialParseConfigFile(ctx, pctx.WithDecodeList(TerraformSource), l, configPath, nil)
 	if err != nil {
 		return false, "", err
 	}
@@ -942,10 +947,10 @@ func terragruntAlreadyInit(l log.Logger, opts *options.TerragruntOptions, config
 
 // getTerragruntOutputJSONFromInitFolder will retrieve the outputs directly from the module's working directory without
 // running init.
-func getTerragruntOutputJSONFromInitFolder(ctx *ParsingContext, l log.Logger, terraformWorkingDir string, iamRoleOpts options.IAMRoleOptions) ([]byte, error) {
-	targetConfigPath := ctx.TerragruntOptions.TerragruntConfigPath
+func getTerragruntOutputJSONFromInitFolder(ctx context.Context, pctx *ParsingContext, l log.Logger, terraformWorkingDir string, iamRoleOpts options.IAMRoleOptions) ([]byte, error) {
+	targetConfigPath := pctx.TerragruntOptions.TerragruntConfigPath
 
-	targetTGOptions, err := setupTerragruntOptionsForBareTerraform(ctx, l, terraformWorkingDir, targetConfigPath, iamRoleOpts)
+	targetTGOptions, err := setupTerragruntOptionsForBareTerraform(ctx, pctx, l, terraformWorkingDir, targetConfigPath, iamRoleOpts)
 	if err != nil {
 		return nil, err
 	}
@@ -976,7 +981,8 @@ func getTerragruntOutputJSONFromInitFolder(ctx *ParsingContext, l log.Logger, te
 // - Clean up folder once json file is generated
 // NOTE: terragruntOptions should be in the ctx of the targetConfig already.
 func getTerragruntOutputJSONFromRemoteState(
-	ctx *ParsingContext,
+	ctx context.Context,
+	pctx *ParsingContext,
 	l log.Logger,
 	targetConfigPath string,
 	remoteState *remotestate.RemoteState,
@@ -986,11 +992,11 @@ func getTerragruntOutputJSONFromRemoteState(
 	// Create working directory where we will run terraform in. We will create the temporary directory in the download
 	// directory for consistency with other file generation capabilities of terragrunt. Make sure it is cleaned up
 	// before the function returns.
-	if err := util.EnsureDirectory(ctx.TerragruntOptions.DownloadDir); err != nil {
+	if err := util.EnsureDirectory(pctx.TerragruntOptions.DownloadDir); err != nil {
 		return nil, err
 	}
 
-	tempWorkDir, err := os.MkdirTemp(ctx.TerragruntOptions.DownloadDir, "")
+	tempWorkDir, err := os.MkdirTemp(pctx.TerragruntOptions.DownloadDir, "")
 	if err != nil {
 		return nil, err
 	}
@@ -1004,15 +1010,15 @@ func getTerragruntOutputJSONFromRemoteState(
 
 	l.Debugf("Setting dependency working directory to %s", tempWorkDir)
 
-	targetTGOptions, err := setupTerragruntOptionsForBareTerraform(ctx, l, tempWorkDir, targetConfigPath, iamRoleOpts)
+	targetTGOptions, err := setupTerragruntOptionsForBareTerraform(ctx, pctx, l, tempWorkDir, targetConfigPath, iamRoleOpts)
 	if err != nil {
 		return nil, err
 	}
 
-	ctx = ctx.WithTerragruntOptions(targetTGOptions)
+	pctx = pctx.WithTerragruntOptions(targetTGOptions)
 
 	// To speed up dependencies processing it is possible to retrieve its output directly from the backend without init dependencies
-	if ctx.TerragruntOptions.Experiments.Evaluate(experiment.DependencyFetchOutputFromState) && !ctx.TerragruntOptions.NoDependencyFetchOutputFromState {
+	if pctx.TerragruntOptions.Experiments.Evaluate(experiment.DependencyFetchOutputFromState) && !pctx.TerragruntOptions.NoDependencyFetchOutputFromState {
 		switch backend := remoteState.BackendName; backend {
 		case s3backend.BackendName:
 			jsonBytes, s3GetErr := getTerragruntOutputJSONFromRemoteStateS3(
@@ -1049,15 +1055,15 @@ func getTerragruntOutputJSONFromRemoteState(
 	l.Debugf("Generated remote state configuration in working dir %s", tempWorkDir)
 
 	// Check for a provider lock file and copy it to the working dir if it exists.
-	terragruntDir := filepath.Dir(ctx.TerragruntOptions.TerragruntConfigPath)
-	if err := CopyLockFile(l, ctx.TerragruntOptions, terragruntDir, tempWorkDir); err != nil {
+	terragruntDir := filepath.Dir(pctx.TerragruntOptions.TerragruntConfigPath)
+	if err := CopyLockFile(l, pctx.TerragruntOptions, terragruntDir, tempWorkDir); err != nil {
 		return nil, err
 	}
 
 	// The working directory is now set up to interact with the state, so pull it down to get the json output.
 
 	// First run init to setup the backend configuration so that we can run output.
-	if err := runTerraformInitForDependencyOutput(ctx, l, tempWorkDir, targetConfigPath); err != nil {
+	if err := runTerraformInitForDependencyOutput(ctx, pctx, l, tempWorkDir, targetConfigPath); err != nil {
 		return nil, err
 	}
 
@@ -1075,7 +1081,7 @@ func getTerragruntOutputJSONFromRemoteState(
 }
 
 // getTerragruntOutputJSONFromRemoteStateS3 pulls the output directly from an S3 bucket without calling Terraform
-func getTerragruntOutputJSONFromRemoteStateS3(ctx *ParsingContext, l log.Logger, opts *options.TerragruntOptions, remoteState *remotestate.RemoteState) ([]byte, error) {
+func getTerragruntOutputJSONFromRemoteStateS3(ctx context.Context, l log.Logger, opts *options.TerragruntOptions, remoteState *remotestate.RemoteState) ([]byte, error) {
 	l.Debugf("Fetching outputs directly from s3://%s/%s", remoteState.BackendConfig["bucket"], remoteState.BackendConfig["key"])
 
 	s3ConfigExtended, err := s3backend.Config(remoteState.BackendConfig).ParseExtendedS3Config()
@@ -1090,7 +1096,7 @@ func getTerragruntOutputJSONFromRemoteStateS3(ctx *ParsingContext, l log.Logger,
 		return nil, errors.New(err)
 	}
 
-	result, err := s3Client.GetObject(ctx.Context, &s3.GetObjectInput{
+	result, err := s3Client.GetObject(ctx, &s3.GetObjectInput{
 		Bucket: aws.String(fmt.Sprintf("%s", remoteState.BackendConfig["bucket"])),
 		Key:    aws.String(fmt.Sprintf("%s", remoteState.BackendConfig["key"])),
 	})
@@ -1128,19 +1134,19 @@ func getTerragruntOutputJSONFromRemoteStateS3(ctx *ParsingContext, l log.Logger,
 
 // setupTerragruntOptionsForBareTerraform sets up a new TerragruntOptions struct that can be used to run terraform
 // without going through the full RunTerragrunt operation.
-func setupTerragruntOptionsForBareTerraform(ctx *ParsingContext, l log.Logger, workingDir string, configPath string, iamRoleOpts options.IAMRoleOptions) (*options.TerragruntOptions, error) {
+func setupTerragruntOptionsForBareTerraform(ctx context.Context, pctx *ParsingContext, l log.Logger, workingDir string, configPath string, iamRoleOpts options.IAMRoleOptions) (*options.TerragruntOptions, error) {
 	// Here we clone the terragrunt options again since we need to make further modifications to it to allow running
 	// terraform directly.
 	// Set the terraform working dir to the tempdir, and set stdout writer to io.Discard so that output content is
 	// not logged.
-	l, targetTGOptions, err := cloneTerragruntOptionsForDependency(ctx, l, configPath)
+	l, targetTGOptions, err := cloneTerragruntOptionsForDependency(pctx, l, configPath)
 	if err != nil {
 		return nil, err
 	}
 
 	targetTGOptions.WorkingDir = workingDir
 	targetTGOptions.Writer = io.Discard
-	targetTGOptions.Engine = ctx.TerragruntOptions.Engine
+	targetTGOptions.Engine = pctx.TerragruntOptions.Engine
 
 	// If the target config has an IAM role directive and it was not set on the command line, set it to
 	// the one we retrieved from the config.
@@ -1159,20 +1165,20 @@ func setupTerragruntOptionsForBareTerraform(ctx *ParsingContext, l log.Logger, w
 
 // runTerragruntOutputJSON uses terragrunt running functions to extract the json output from the target config.
 // NOTE: targetTGOptions should be in the ctx of the targetConfig.
-func runTerragruntOutputJSON(ctx *ParsingContext, l log.Logger, targetConfig string) ([]byte, error) {
+func runTerragruntOutputJSON(ctx context.Context, pctx *ParsingContext, l log.Logger, targetConfig string) ([]byte, error) {
 	// Update the stdout buffer so we can capture the output
 	var stdoutBuffer bytes.Buffer
 
 	stdoutBufferWriter := bufio.NewWriter(&stdoutBuffer)
 
-	newOpts := *ctx.TerragruntOptions
+	newOpts := *pctx.TerragruntOptions
 	// explicit disable json formatting and prefixing to read json output
 	newOpts.ForwardTFStdout = false
 	newOpts.JSONLogFormat = false
 	newOpts.Writer = stdoutBufferWriter
-	ctx = ctx.WithTerragruntOptions(&newOpts)
+	pctx = pctx.WithTerragruntOptions(&newOpts)
 
-	err := ctx.TerragruntOptions.RunTerragrunt(ctx, l, ctx.TerragruntOptions, report.NewReport())
+	err := pctx.TerragruntOptions.RunTerragrunt(ctx, l, pctx.TerragruntOptions, report.NewReport())
 	if err != nil {
 		return nil, errors.New(err)
 	}
@@ -1238,10 +1244,10 @@ func ClearOutputCache() {
 // plugins are available, even though we don't need it for our purposes (terraform output does not depend on any of the
 // plugins being available). As such this command will ignore errors in the init command.
 // To help with debuggability, the errors will be printed to the console when TG_LOG=debug is set.
-func runTerraformInitForDependencyOutput(ctx *ParsingContext, l log.Logger, workingDir string, targetConfigPath string) error {
+func runTerraformInitForDependencyOutput(ctx context.Context, pctx *ParsingContext, l log.Logger, workingDir string, targetConfigPath string) error {
 	stderr := bytes.Buffer{}
 
-	l, initTGOptions, err := cloneTerragruntOptionsForDependency(ctx, l, targetConfigPath)
+	l, initTGOptions, err := cloneTerragruntOptionsForDependency(pctx, l, targetConfigPath)
 	if err != nil {
 		return err
 	}
