@@ -20,6 +20,7 @@ import (
 	"github.com/gruntwork-io/terragrunt/internal/cache"
 	"github.com/gruntwork-io/terragrunt/internal/experiment"
 	"github.com/gruntwork-io/terragrunt/internal/github"
+	"github.com/gruntwork-io/terragrunt/internal/os/signal"
 
 	"github.com/hashicorp/go-hclog"
 
@@ -445,8 +446,16 @@ func Shutdown(ctx context.Context, l log.Logger, opts *options.TerragruntOptions
 	engineClients.Range(func(key, value any) bool {
 		instance := value.(*engineInstance)
 		l.Debugf("Shutting down engine for %s", instance.executionOptions.WorkingDir)
-		// invoke shutdown on engine
-		if err := shutdown(ctx, l, instance.executionOptions, instance.terragruntEngine); err != nil {
+
+		// We use without cancel here to ensure that the shutdown isn't cancelled by the main context,
+		// like it is in the RunCommandWithOutput function. This ensures that we don't cancel the shutdown
+		// when the command is cancelled.
+		if err := shutdown(
+			context.WithoutCancel(ctx),
+			l,
+			instance.executionOptions,
+			instance.terragruntEngine,
+		); err != nil {
 			l.Errorf("Error shutting down engine: %v", err)
 		}
 		// kill grpc client
@@ -500,7 +509,24 @@ func createEngine(ctx context.Context, l log.Logger, terragruntOptions *options.
 		Output: l.Writer(),
 	})
 
-	cmd := exec.CommandContext(ctx, localEnginePath)
+	// We use without cancel here to ensure that the plugin isn't killed when the main context is cancelled,
+	// like it is in the RunCommandWithOutput function. This ensures that we don't cancel the shutdown
+	// when the command is cancelled.
+	cmd := exec.CommandContext(
+		context.WithoutCancel(ctx),
+		localEnginePath,
+	)
+	cmd.Cancel = func() error {
+		if cmd.Process == nil {
+			return nil
+		}
+
+		if sig := signal.SignalFromContext(ctx); sig != nil {
+			return cmd.Process.Signal(sig)
+		}
+
+		return cmd.Process.Signal(os.Kill)
+	}
 	// pass log level to engine
 	cmd.Env = append(cmd.Env, fmt.Sprintf("%s=%s", engineLogLevelEnv, engineLogLevel))
 	client := plugin.NewClient(&plugin.ClientConfig{
@@ -743,7 +769,6 @@ func ReadEngineOutput(runOptions *ExecutionOptions, forceStdErr bool, output out
 
 	for {
 		response, err := output()
-
 		if err != nil && (errors.Is(err, ErrEngineInitFailed) || errors.Is(err, ErrEngineShutdownFailed)) {
 			return err
 		}
