@@ -13,7 +13,6 @@ import (
 	"slices"
 	"strings"
 
-	"github.com/gruntwork-io/go-commons/collections"
 	"github.com/gruntwork-io/terragrunt/internal/cli"
 
 	"github.com/gruntwork-io/terragrunt/tf"
@@ -208,6 +207,23 @@ func resolveUnitsFromDiscovery(
 			continue
 		}
 
+		// Transfer discovery context command and args to unit options if available
+		if discoveryCtx := unit.DiscoveryContext(); discoveryCtx != nil {
+			if discoveryCtx.Cmd != "" {
+				unitOpts.TerraformCommand = discoveryCtx.Cmd
+			}
+
+			if len(discoveryCtx.Args) > 0 {
+				terraformCliArgs := make([]string, 0, 1+len(discoveryCtx.Args))
+				if discoveryCtx.Cmd != "" {
+					terraformCliArgs = append(terraformCliArgs, discoveryCtx.Cmd)
+				}
+
+				terraformCliArgs = append(terraformCliArgs, discoveryCtx.Args...)
+				unitOpts.TerraformCliArgs = terraformCliArgs
+			}
+		}
+
 		// Initialize execution context
 		unit.Execution = &component.UnitExecution{
 			TerragruntOptions:    unitOpts,
@@ -435,7 +451,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 	}
 
 	if slices.Contains(config.TerraformCommandsNeedInput, terraformCmd) {
-		opts.TerraformCliArgs = util.StringListInsert(opts.TerraformCliArgs, "-input=false", 1)
+		opts.TerraformCliArgs = slices.Insert(opts.TerraformCliArgs, 1, "-input=false")
 		r.syncTerraformCliArgs(l, opts)
 	}
 
@@ -444,7 +460,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 	switch terraformCmd {
 	case tf.CommandNameApply, tf.CommandNameDestroy:
 		if opts.RunAllAutoApprove {
-			opts.TerraformCliArgs = util.StringListInsert(opts.TerraformCliArgs, "-auto-approve", 1)
+			opts.TerraformCliArgs = slices.Insert(opts.TerraformCliArgs, 1, "-auto-approve")
 		}
 
 		r.syncTerraformCliArgs(l, opts)
@@ -464,7 +480,13 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 				// Ensure path is absolute for reporting
 				unitPath := u.AbsolutePath()
 
-				run, err := r.Stack.Execution.Report.EnsureRun(l, unitPath)
+				// Pass the discovery working directory for worktree scenarios
+				var ensureOpts []report.EndOption
+				if discoveryCtx := u.DiscoveryContext(); discoveryCtx != nil && discoveryCtx.WorkingDir != "" {
+					ensureOpts = append(ensureOpts, report.WithDiscoveryWorkingDir(discoveryCtx.WorkingDir))
+				}
+
+				run, err := r.Stack.Execution.Report.EnsureRun(l, unitPath, ensureOpts...)
 				if err != nil {
 					l.Errorf("Error ensuring run for unit %s: %v", unitPath, err)
 					continue
@@ -553,7 +575,13 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 				// Ensure path is absolute for reporting
 				unitPath := unit.AbsolutePath()
 
-				run, reportErr := r.Stack.Execution.Report.EnsureRun(l, unitPath)
+				// Pass the discovery working directory for worktree scenarios
+				var ensureOpts []report.EndOption
+				if discoveryCtx := unit.DiscoveryContext(); discoveryCtx != nil && discoveryCtx.WorkingDir != "" {
+					ensureOpts = append(ensureOpts, report.WithDiscoveryWorkingDir(discoveryCtx.WorkingDir))
+				}
+
+				run, reportErr := r.Stack.Execution.Report.EnsureRun(l, unitPath, ensureOpts...)
 				if reportErr != nil {
 					l.Errorf("Error ensuring run for early exit unit %s: %v", unitPath, reportErr)
 					continue
@@ -732,7 +760,24 @@ func (r *Runner) syncTerraformCliArgs(l log.Logger, opts *options.TerragruntOpti
 			continue
 		}
 
-		unit.Execution.TerragruntOptions.TerraformCliArgs = collections.MakeCopyOfList(opts.TerraformCliArgs)
+		discoveryCtx := unit.DiscoveryContext()
+		if discoveryCtx != nil && len(discoveryCtx.Args) > 0 {
+			mergedArgs := slices.Clone(unit.Execution.TerragruntOptions.TerraformCliArgs)
+
+			for _, stackArg := range opts.TerraformCliArgs {
+				if stackArg == opts.TerraformCliArgs.First() {
+					continue
+				}
+
+				if !slices.Contains(mergedArgs, stackArg) {
+					mergedArgs = append(mergedArgs, stackArg)
+				}
+			}
+
+			unit.Execution.TerragruntOptions.TerraformCliArgs = mergedArgs
+		} else {
+			unit.Execution.TerragruntOptions.TerraformCliArgs = slices.Clone(opts.TerraformCliArgs)
+		}
 
 		planFile := unit.PlanFile(opts)
 		if planFile != "" {
