@@ -87,7 +87,7 @@ func EnsureDirectory(path string) error {
 // components (e.g. "../") fully resolved, which makes it safe to compare paths as strings.
 func CanonicalPath(path string, basePath string) (string, error) {
 	if !filepath.IsAbs(path) {
-		path = JoinPath(basePath, path)
+		path = filepath.Join(basePath, path)
 	}
 
 	absPath, err := filepath.Abs(path)
@@ -440,6 +440,10 @@ func CopyFolderContents(
 	includeInCopy []string,
 	excludeFromCopy []string,
 ) error {
+	// We use filepath.ToSlash because we end up using globs here, and those expect forward slashes.
+	source = filepath.ToSlash(source)
+	destination = filepath.ToSlash(destination)
+
 	// Expand all the includeInCopy glob paths, converting the globbed results to relative paths so that they work in
 	// the copy filter.
 	includeExpandedGlobs := []string{}
@@ -618,15 +622,6 @@ func WriteFileWithSamePermissions(source string, destination string, contents []
 	return os.WriteFile(destination, contents, fileInfo.Mode())
 }
 
-// JoinPath is a wrapper around filepath.Join
-//
-// Windows systems use \ as the path separator *nix uses /
-// Use this function when joining paths to force the returned path to use / as the path separator
-// This will improve cross-platform compatibility
-func JoinPath(elem ...string) string {
-	return filepath.ToSlash(filepath.Join(elem...))
-}
-
 // SplitPath splits the given path into a list.
 // E.g. "foo/bar/boo.txt" -> ["foo", "bar", "boo.txt"]
 // E.g. "/foo/bar/boo.txt" -> ["", "foo", "bar", "boo.txt"]
@@ -831,17 +826,17 @@ func (err PathIsNotFile) Error() string {
 func ListTfFiles(directoryPath string, walkWithSymlinks bool) ([]string, error) {
 	var tfFiles []string
 
-	walkFunc := filepath.Walk
+	walkFunc := filepath.WalkDir
 	if walkWithSymlinks {
-		walkFunc = WalkWithSymlinks
+		walkFunc = WalkDirWithSymlinks
 	}
 
-	err := walkFunc(directoryPath, func(path string, info os.FileInfo, err error) error {
+	err := walkFunc(directoryPath, func(path string, d fs.DirEntry, err error) error {
 		if err != nil {
 			return err
 		}
 
-		if !info.IsDir() && filepath.Ext(path) == TfFileExtension {
+		if !d.IsDir() && filepath.Ext(path) == TfFileExtension {
 			tfFiles = append(tfFiles, path)
 		}
 
@@ -901,8 +896,11 @@ func GetTempDir() (string, error) {
 	return tempDir, nil
 }
 
-// GetExcludeDirsFromFile returns a list of directories from the given filename, where each directory path starts on a new line.
-func GetExcludeDirsFromFile(baseDir, filename string) ([]string, error) {
+// ExcludeFiltersFromFile returns a list of filters from the given filename, where each filter starts on a new line.
+//
+// Note that this is a backwards compatibility implementation for the `--queue-excludes-file` flag, so it's going to
+// append the ! prefix to each filter to negate it.
+func ExcludeFiltersFromFile(baseDir, filename string) ([]string, error) {
 	filename, err := CanonicalPath(filename, baseDir)
 	if err != nil {
 		return nil, err
@@ -918,8 +916,8 @@ func GetExcludeDirsFromFile(baseDir, filename string) ([]string, error) {
 	}
 
 	var (
-		lines = strings.Split(strings.ReplaceAll(content, "\r\n", "\n"), "\n")
-		dirs  = make([]string, 0, len(lines))
+		lines   = strings.Split(strings.ReplaceAll(content, "\r\n", "\n"), "\n")
+		filters = make([]string, 0, len(lines))
 	)
 
 	for _, dir := range lines {
@@ -928,10 +926,43 @@ func GetExcludeDirsFromFile(baseDir, filename string) ([]string, error) {
 			continue
 		}
 
-		dirs = append(dirs, dir)
+		filters = append(filters, "!"+dir)
 	}
 
-	return dirs, nil
+	return filters, nil
+}
+
+// GetFiltersFromFile returns a list of filter queries from the given filename, where each filter query starts on a new line.
+func GetFiltersFromFile(baseDir, filename string) ([]string, error) {
+	filename, err := CanonicalPath(filename, baseDir)
+	if err != nil {
+		return nil, err
+	}
+
+	if !FileExists(filename) || !IsFile(filename) {
+		return nil, nil
+	}
+
+	content, err := ReadFileAsString(filename)
+	if err != nil {
+		return nil, err
+	}
+
+	var (
+		lines   = strings.Split(strings.ReplaceAll(content, "\r\n", "\n"), "\n")
+		filters = make([]string, 0, len(lines))
+	)
+
+	for _, filter := range lines {
+		filter = strings.TrimSpace(filter)
+		if filter == "" || strings.HasPrefix(filter, "#") {
+			continue
+		}
+
+		filters = append(filters, filter)
+	}
+
+	return filters, nil
 }
 
 // MatchSha256Checksum returns the SHA256 checksum for the given file and filename.
@@ -1023,107 +1054,6 @@ func Copy(ctx context.Context, dst io.Writer, src io.Reader) (int64, error) {
 	return num, err
 }
 
-// WalkWithSymlinks traverses a directory tree, following symbolic links and calling
-// the provided function for each file or directory encountered. It handles both regular
-// symlinks and circular symlinks without getting into infinite loops.
-//
-//nolint:funlen
-func WalkWithSymlinks(root string, externalWalkFn filepath.WalkFunc) error {
-	// pathPair keeps track of both the physical (real) path on disk
-	// and the logical path (how it appears in the walk)
-	type pathPair struct {
-		physical string
-		logical  string
-	}
-
-	// visited tracks symlink paths to prevent circular references
-	// key is combination of realPath:symlinkPath
-	visited := make(map[string]bool)
-
-	// visitedLogical tracks logical paths to prevent duplicates
-	// when the same directory is reached through different symlinks
-	visitedLogical := make(map[string]bool)
-
-	var walkFn func(pathPair) error
-
-	walkFn = func(pair pathPair) error {
-		return filepath.Walk(pair.physical, func(currentPath string, info os.FileInfo, err error) error {
-			if err != nil {
-				return externalWalkFn(currentPath, info, err)
-			}
-
-			// Convert the current physical path to a logical path relative to the walk root
-			rel, err := filepath.Rel(pair.physical, currentPath)
-			if err != nil {
-				return errors.Errorf("failed to get relative path between %s and %s: %w", pair.physical, currentPath, err)
-			}
-
-			logicalPath := filepath.Join(pair.logical, rel)
-
-			realPath, realInfo, err := evalRealPathAndInfo(currentPath)
-			if err != nil {
-				return err
-			}
-
-			// Call the provided function only if we haven't seen this logical path before
-			if !visitedLogical[logicalPath] {
-				visitedLogical[logicalPath] = true
-
-				if err := externalWalkFn(logicalPath, realInfo, nil); err != nil {
-					return err
-				}
-			}
-
-			// If we encounter a symlink, resolve and follow it
-			if info.Mode()&os.ModeSymlink != 0 {
-				// Skip if we've seen this symlink->target combination before
-				// This prevents infinite loops with circular symlinks
-				if visited[realPath+":"+currentPath] {
-					return nil
-				}
-
-				visited[realPath+":"+currentPath] = true
-
-				// If the target is a directory, recursively walk it
-				if realInfo.IsDir() {
-					return walkFn(pathPair{
-						physical: realPath,
-						logical:  logicalPath,
-					})
-				}
-			}
-
-			return nil
-		})
-	}
-
-	realRoot, err := filepath.EvalSymlinks(root)
-	if err != nil {
-		return errors.Errorf("failed to get evaluate sym links for %s: %w", root, err)
-	}
-
-	// Start the walk from the root directory
-	return walkFn(pathPair{
-		physical: realRoot,
-		logical:  realRoot,
-	})
-}
-
-func evalRealPathAndInfo(currentPath string) (string, os.FileInfo, error) {
-	realPath, err := filepath.EvalSymlinks(currentPath)
-	if err != nil {
-		return "", nil, errors.Errorf("failed to get evaluate sym links for %s: %w", currentPath, err)
-	}
-
-	// Get info about the symlink target
-	realInfo, err := os.Stat(realPath)
-	if err != nil {
-		return "", nil, errors.Errorf("failed to describe file %s: %w", realPath, err)
-	}
-
-	return realPath, realInfo, nil
-}
-
 // evalRealPathForWalkDir evaluates symlinks and returns the real path and whether it's a directory.
 func evalRealPathForWalkDir(currentPath string) (string, bool, error) {
 	realPath, err := filepath.EvalSymlinks(currentPath)
@@ -1143,9 +1073,6 @@ func evalRealPathForWalkDir(currentPath string) (string, bool, error) {
 // and calling the provided function for each file or directory encountered. It handles both regular
 // symlinks and circular symlinks without getting into infinite loops.
 //
-// This function is similar to WalkWithSymlinks but uses the newer filepath.WalkDir API which
-// accepts fs.DirEntry instead of os.FileInfo.
-//
 //nolint:funlen
 func WalkDirWithSymlinks(root string, externalWalkFn fs.WalkDirFunc) error {
 	// pathPair keeps track of both the physical (real) path on disk
